from . import console
from ._print import builtin_print
from ._print import builtin_print as bprint  # alias
from .control import disable
from .control import enable
from .control import setup
from .control import start_ipython
from .control import unload
from .control import update
from .logger import lk

__version__ = '5.3.2'
